#pragma title ("CWMQM - CW MQM Routines")
#pragma page ()

#ifndef __CW_MQM
   #define __CW_MQM 1 /* Indicate that this file has been included. */

       /*----------------------------------------------------+
       | CHANGE HISTORY for CW MQM interface routines.       |
       | Change codes are written in column 73-78.           |
       +-----------------------------------------------------+
       | Date       Change-id   Produced     SCR number.     |
       | yyyymmdd               Version.                     |
       |                        rr.ss.uu                     |
       | Reason                                              |
       +-----------------------------------------------------+
       | 19980601   R. Lacroix  01.00.00                     |
       | Birth.                                              |
       |                                                     |
       |                                                     |
       +----------------------------------------------------*/

       /*----------------------------------------------------+
       | Purpose:  These routines will be used to access     |
       |           MQM services.                             |
       |                                                     |
       | Functions:                                          |
       |           CW_ConnectToMqm                           |
       |           CW_OpenMqmQueue                           |
       |           CW_CloseMqmQueue                          |
       |           CW_GetMqmMessage                          |
       |           CW_SendMqmMessage                         |
       |           CW_DisconnectFromMqm                      |
       |                                                     |
       +----------------------------------------------------*/

#pragma subtitle ("CW_ConnectToMqm")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_ConnectToMqm                            |
       |                                                     |
       | Purpose: This routine will establish a connection   |
       | between us and the local MQM. If it fails, put out  |
       | a message and return a non-zero return code.        |
       |                                                     |
       | Input:                                              |
       |  QueueMgrName - MQM image to be connected to.       |
       |                                                     |
       | Output:                                             |
       |  LocalMqmHandle - the handle that will be associated|
       |                   with this connection.             |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed.                |
       |                  No connection is established.      |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQCONN                                             |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_ConnectToMqm( QUEUE_NAME   QueueMgrName )

{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T  CW_RC;        /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_ConnectToMqm\n");

   /*--------------------------------------------------------+
   | Call MQCONN to connect to the local queue manager.      |
   +--------------------------------------------------------*/

   MQCONN( QueueMgrName,
           &LocalMqmHandle,
           &MqmReturnCode ,
           &MqmReasonCode );

   /*--------------------------------------------------------+
   | If the connection failed, tell the user.                |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      printf( "MQConn failed with Return Code = %d and Reason "\
              "Code = %d\n",
              MqmReturnCode,
              MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   return ( CW_RC );
}


#pragma subtitle ("CW_OpenMqmQueue")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_OpenMqmQueue                            |
       |                                                     |
       | Purpose: This routine will open the named queue.    |
       |                                                     |
       | Input:                                              |
       |  QueueName   - the name of the queue to be opened.  |
       |  OpenOptions - the MQM options for this open.       |
       |                                                     |
       | Output:                                             |
       |  QueueHandlePtr - pointer to the handle that will   |
       |                   be associated with this queue.    |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed. The queue      |
       |                  was not opened.                    |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQOPEN                                             |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_OpenMqmQueue( QUEUE_NAME     QueueMgrName,
                                QUEUE_NAME     QueueName,
                                QUEUE_NAME     DynamicQueueName,
                                MQHOBJ        *QueueHandlePtr,
                                MQLONG         OpenOptions )
{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;         /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   auto   MQOD      ObjDesc = { MQOD_DEFAULT };   /* MQOD block   */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_OpenMqmQueue\n");

   /*------------------------------------------------------+
   | Initialize the target queue manager to be the one     |
   | passed to us.                                         |
   +------------------------------------------------------*/

   memcpy (ObjDesc.ObjectQMgrName,
           QueueMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   /*------------------------------------------------------+
   | Initialize the object name to the passed queue name.  |
   +------------------------------------------------------*/

   memcpy( ObjDesc.ObjectName,
           QueueName,
           MQ_Q_NAME_LENGTH );

   /*------------------------------------------------------+
   | Initialize the dynamic queue name to the passed name. |
   +------------------------------------------------------*/

   memcpy (ObjDesc.DynamicQName,
           DynamicQueueName,
           MQ_Q_NAME_LENGTH);

   /*--------------------------------------------------------+
   | Call MQOPEN to open the named queue.                    |
   +--------------------------------------------------------*/

   MQOPEN( LocalMqmHandle,
           &ObjDesc,
           OpenOptions,
           QueueHandlePtr,
           &MqmReturnCode,
           &MqmReasonCode );

   /*------------------------------------------------------+
   | Copy the resolved queue name to the field passed by   |
   | the caller.                                           |
   +------------------------------------------------------*/

   memcpy (QueueMgrName,
           ObjDesc.ObjectQMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   memcpy (QueueName,
           ObjDesc.ObjectName,
           MQ_Q_NAME_LENGTH);

   printf("Open a Queue: QueueMgrName = %-48.48s\n", QueueMgrName);
   printf("              QueueName    = %-48.48s\n", QueueName);

   /*--------------------------------------------------------+
   | If the open failed, tell the user.                      |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      printf( "MQOpen failed with Return Code = %d and Reason "\
              "Code = %d\n",
              MqmReturnCode,
              MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   return ( CW_RC );
}


#pragma subtitle ("CW_CloseMqmQueue")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_CloseMqmQueue                           |
       |                                                     |
       | Purpose: This routine will close the queue          |
       |          represented by the passed handle.          |
       |                                                     |
       | Input:                                              |
       |  QueueHandlePtr - pointer to the handle associated  |
       |                   with the queue to be closed.      |
       |                                                     |
       | Output:                                             |
       |  None.                                              |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed. The queue      |
       |                  close failed.                      |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQCLOSE                                            |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_CloseMqmQueue( MQHOBJ      *QueueHandlePtr )
{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;         /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_CloseMqmQueue\n");

   /*--------------------------------------------------------+
   | Call MQCLOSE to close the named queue.                  |
   +--------------------------------------------------------*/

   MQCLOSE( LocalMqmHandle,
            QueueHandlePtr,
            MQCO_NONE,
            &MqmReturnCode,
            &MqmReasonCode );

   /*--------------------------------------------------------+
   | If the close failed, tell the user.                     |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      printf( "MQClose failed with Return Code = %d and Reason "\
              "Code = %d\n",
              MqmReturnCode,
              MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   return ( CW_RC );
}


#pragma subtitle ("CW_GetMqmMessage")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_GetMqmMessage                           |
       |                                                     |
       | Purpose: This routine will get a message from the   |
       |          named queue.                               |
       |                                                     |
       | Input:                                              |
       |  QueueHandlePtr - pointer to the handle associated  |
       |                   with the queue to be closed.      |
       |  ReceiveMsgPtr  - pointer to the receive data block.|
       |  ReceiveMsgSize - size of the receive data blcok.   |
       |  MqmTimeOut     - number of seconds to wait for a   |
       |                   message.                          |
       |  ReturnedDataLength - size of the message returned. |
       |                                                     |
       | Output:                                             |
       |  ReturnedQueueName - The queue that is to be used   |
       |                      to send a reply.               |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_WARNING- There was nothing on the queue.     |
       |                                                     |
       |  CW_RC_FAILED - The function failed. The GET did    |
       |                  not complete.                      |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQGET                                              |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_GetMqmMessage(  MQHOBJ         QueueHandlePtr,
                                  void          *ReceiveMsgPtr,
                                  int            ReceiveMsgSize,
                                  int            MqmTimeOut,
                                  MQLONG        *ReturnedDataLength,
                                  CW_REPLYTO_T  *ReplyTo )

{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;        /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   auto   MQGMO     GetMsgOpts = { MQGMO_DEFAULT };/* MQGMO block */

   auto   MQMD      MsgDesc = { MQMD_DEFAULT };   /* MQMD block   */

   auto   flag_type CommitRequired = OFF;

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_GetMqmMessage\n");

   /*------------------------------------------------------+
   | Initialize the get message options.                   |
   +------------------------------------------------------*/

   if ( MqmTimeOut == 0 )
   {
      GetMsgOpts.WaitInterval = MQWI_UNLIMITED;
   }
   else
   {
      GetMsgOpts.WaitInterval = MqmTimeOut * 1000;      /* in ms */
   }

   GetMsgOpts.Options = MQGMO_WAIT  +
                        MQGMO_ACCEPT_TRUNCATED_MSG +
                        MQGMO_FAIL_IF_QUIESCING +
                        MQGMO_SYNCPOINT;

   /*------------------------------------------------------+
   | Initialize the message descriptor. Set the reporting  |
   | to none.                                              |
   +------------------------------------------------------*/

   MsgDesc.Report = MQRO_NONE;

   /*------------------------------------------------------+
   | Set the message id and correlation id to none. We just|
   | want the first one off the queue.                     |
   +------------------------------------------------------*/

   memcpy( MsgDesc.MsgId,
           MQMI_NONE,
           sizeof(MsgDesc.MsgId) );

   memcpy( MsgDesc.CorrelId,
           MQCI_NONE,
           sizeof(MsgDesc.CorrelId) );

   /*--------------------------------------------------------+
   | Call MQGET to get the first message from the queue, or  |
   | register an ECB with MQM.                               |
   +--------------------------------------------------------*/

   MQGET( LocalMqmHandle,
          QueueHandlePtr,
          &MsgDesc,
          &GetMsgOpts,
          ReceiveMsgSize,
          ReceiveMsgPtr,
          ReturnedDataLength,
          &MqmReturnCode,
          &MqmReasonCode );

   printf( "After MQGET - Mqm Return code =%d - Reason code %d\n",
           MqmReturnCode,
           MqmReasonCode );

   /*--------------------------------------------------------+
   | Examine the return code.                                |
   +--------------------------------------------------------*/

   switch ( MqmReturnCode )
   {
      case MQCC_OK:

         /*--------------------------------------------------+
         | MqmReturnCode = MQCC_OK                           |
         |                                                   |
         | A message was retrieved. Set the function return  |
         | code to zero and get lost.                        |
         +--------------------------------------------------*/

         CW_RC = CW_RC_OK;

         /*--------------------------------------------------+
         | Ask for a commit to happen since we have          |
         | successfully retrieved the message.               |
         +--------------------------------------------------*/

         CommitRequired = ON;

         break;

      case MQCC_WARNING:

         /*--------------------------------------------------+
         | MqmReturnCode = MQCC_WARNING.                     |
         |                                                   |
         | The message we received had to be truncated.      |
         +--------------------------------------------------*/

         if (MqmReasonCode == MQRC_TRUNCATED_MSG_ACCEPTED)
         {
            /*-----------------------------------------------+
            | Since we do not want to deal with the          |
            | truncated message again, ask for a commit.     |
            +-----------------------------------------------*/

            CommitRequired = ON;

            printf( "Warning: MQGet failed with Return Code = %d "\
                    "and Reason Code = %d\n",
                    MqmReturnCode,
                    MqmReasonCode );

            CW_RC = CW_RC_WARNING;

         }
         else
         {
            printf( "MQGet failed with Return Code = %d and Reason "\
                    "Code = %d\n",
                    MqmReturnCode,
                    MqmReasonCode );

            CW_RC = CW_RC_FAILED;
         }

         break;

      case MQCC_FAILED:

         /*--------------------------------------------------+
         | MqmReturnCode = MQCC_FAILED                       |
         |                                                   |
         | The call failed.                                  |
         +--------------------------------------------------*/

         /*--------------------------------------------------------+
         | Set the return and reason code in the API control area. |
         +--------------------------------------------------------*/

         if (MqmReasonCode == MQRC_NO_MSG_AVAILABLE)
         {
            /*-----------------------------------------------+
            | The timeout has expired.  Lets tell the user.  |
            +-----------------------------------------------*/

            printf( "Warning: MQGet failed with Return Code = %d "\
                    "and Reason Code = %d\n",
                    MqmReturnCode,
                    MqmReasonCode );

            CW_RC = CW_RC_WARNING;
         }
         else
         {
            printf( "MQGet failed with Return Code = %d and Reason "\
                    "Code = %d\n",
                    MqmReturnCode,
                    MqmReasonCode );

            CW_RC = CW_RC_FAILED;
         }

         break;

      default:

         /*--------------------------------------------------+
         | default can never happen. Set the return code     |
         | to failed.                                        |
         +--------------------------------------------------*/

         printf( "MQGet failed with Return Code = %d and Reason "\
                 "Code = %d\n",
                 MqmReturnCode,
                 MqmReasonCode );

         CW_RC = CW_RC_FAILED;

         break;

   }

   /*--------------------------------------------------------+
   | Does the user wnat the Reply-To Queue Manager and       |
   | Queue names.  Did they give us a none blank pointer?    |
   +--------------------------------------------------------*/

   if ( ( (MqmReturnCode == MQCC_OK)    ||
          (MqmReturnCode == MQCC_WARNING))  &&
        ( ReplyTo != NULL ) )
   {
      /*------------------------------------------------------+
      | Set the reply-to-queue name to the local MQM queue.   |
      +------------------------------------------------------*/

      memcpy( ReplyTo->QueueMgrName,
              MsgDesc.ReplyToQMgr,
              sizeof(ReplyTo->QueueMgrName) );

      memcpy( ReplyTo->QueueName,
              MsgDesc.ReplyToQ,
              sizeof(ReplyTo->QueueName) );
   }

   /*--------------------------------------------------------+
   | If a message was pulled from the queue, issue a commit  |
   | call to ensure that it is removed permanently.          |
   +--------------------------------------------------------*/

   if ( CommitRequired == ON )
   {

      printf("Now doing the commit.\n");

      MQCMIT( LocalMqmHandle,
              &MqmReturnCode,
              &MqmReasonCode );

      /*-----------------------------------------------------+
      | Examine the return code.                             |
      +-----------------------------------------------------*/

      if ( MqmReturnCode != MQCC_OK )
      {
         /*--------------------------------------------------------+
         | Set the return and reason code in the API control area. |
         +--------------------------------------------------------*/

         printf( "MQCMIT failed with Return Code = %d and Reason "\
                 "Code = %d\n",
                 MqmReturnCode,
                 MqmReasonCode );

         CW_RC = CW_RC_FAILED;
      }

   }  /* end of if return code is OK */

   return ( CW_RC );
}


#pragma subtitle ("CW_SendMqmMessage")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_SendMqmMessage                          |
       |                                                     |
       | Purpose: This routine will send an MQM message to   |
       |          the queue associated with the passed       |
       |          handle. Obviously, the queue is assumed to |
       |          be open.                                   |
       |                                                     |
       | Input:                                              |
       |  QueueHandlePtr - pointer to the handle associated  |
       |                   with the queue to be closed.      |
       |  SendMsgPtr     - pointer to the send data block.   |
       |  SendMsgSize    - size of the send data blcok.      |
       |  ReplyToQueueName- name of YOUR reply queue.        |
       |                                                     |
       | Output:                                             |
       |  None.                                              |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed. The data was   |
       |                  not put onto a queue.              |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQPUT                                              |
       |  MQCMIT                                             |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_SendMqmMessage(  MQHOBJ          QueueHandlePtr,
                                   void           *SendMsgPtr,
                                   int             SendMsgSize,
                                   QUEUE_NAME      ReplyToQueueName)
{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;         /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   auto   MQPMO     PutMsgOpts = { MQPMO_DEFAULT };/* MQPMO block */

   auto   MQMD      MsgDesc = { MQMD_DEFAULT };   /* MQMD block   */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_SendMqmMessage\n");

   /*------------------------------------------------------+
   | Initialize the put message options to send the default|
   | context associated with the queue.                    |
   +------------------------------------------------------*/

   PutMsgOpts.Options = MQPMO_DEFAULT_CONTEXT;

   /*------------------------------------------------------+
   | Initialize the message descriptor. Set the reporting  |
   | to none.                                              |
   +------------------------------------------------------*/

   MsgDesc.Report = MQRO_NONE;

   /*------------------------------------------------------+
   | Set no expiration.                                    |
   +------------------------------------------------------*/

   MsgDesc.Expiry = MQEI_UNLIMITED;

   /*------------------------------------------------------+
   | Set the message type to be a request. This will allow |
   | the sender to determine who sent the request.         |
   +------------------------------------------------------*/

   MsgDesc.MsgType = MQMT_REQUEST;

   /*------------------------------------------------------+
   | default to the persistence and the priority of the    |
   | queue.                                                |
   +------------------------------------------------------*/

   MsgDesc.Persistence = MQPER_PERSISTENCE_AS_Q_DEF;
   MsgDesc.Priority    = MQPRI_PRIORITY_AS_Q_DEF;

   /*------------------------------------------------------+
   | Set the reply-to-queue name to the local MQM queue.   |
   +------------------------------------------------------*/

   memcpy( MsgDesc.ReplyToQ,
           ReplyToQueueName,
           sizeof(MsgDesc.ReplyToQ) );

   /*--------------------------------------------------------+
   | Call MQPUT to put the message onto the queue.           |
   +--------------------------------------------------------*/

   MQPUT( LocalMqmHandle,
          QueueHandlePtr,
          &MsgDesc,
          &PutMsgOpts,
          SendMsgSize,
          SendMsgPtr,
          &MqmReturnCode,
          &MqmReasonCode );

   printf( "After MQPUT - Mqm Return code =%d - Reason code %d\n",
           MqmReturnCode,
           MqmReasonCode );

   /*--------------------------------------------------------+
   | Examine the return code.                                |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      /*--------------------------------------------------------+
      | Set the return and reason code in the API control area. |
      +--------------------------------------------------------*/

      printf( "MQPUT failed with Return Code = %d and Reason Code "\
              "= %d\n",
              MqmReturnCode,
              MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   /*--------------------------------------------------------+
   | If the MQM put worked, issue a commit call to forward   |
   | the message to the target application.                  |
   +--------------------------------------------------------*/

   if ( CW_RC == CW_RC_OK )
   {
      printf("Now doing the commit.\n");

      MQCMIT( LocalMqmHandle,
              &MqmReturnCode,
              &MqmReasonCode );

      /*-----------------------------------------------------+
      | Examine the return code.                             |
      +-----------------------------------------------------*/

      if ( MqmReturnCode != MQCC_OK )
      {
         /*--------------------------------------------------------+
         | Set the return and reason code in the API control area. |
         +--------------------------------------------------------*/

         printf( "MQCMIT failed with Return Code = %d and Reason "\
                 "Code = %d\n",
                 MqmReturnCode,
                 MqmReasonCode );

         CW_RC = CW_RC_FAILED;
      }

   }

   return ( CW_RC );
}


#pragma subtitle ("CW_DisconnectFromMqm")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_DisconnectFromMqm                       |
       |                                                     |
       | Purpose: This routine will sever an existing        |
       | connection between us and MQM. If it fails, put out |
       | a message and return a non-zero return code.        |
       |                                                     |
       | Input:                                              |
       |  None.                                              |
       |                                                     |
       | Output:                                             |
       |  None.                                              |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed. The            |
       |                  connection was not dropped.        |
       |                                                     |
       | Abend Codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQDISC                                             |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_DisconnectFromMqm( )
{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;        /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;        /* Assume that the return code is OK. */

   printf("Now entering CW_DisconnectFromMqm\n");

   /*--------------------------------------------------------+
   | Call MQDISC to disconnect from the local queue manager. |
   +--------------------------------------------------------*/

   MQDISC( &LocalMqmHandle,
           &MqmReturnCode,
           &MqmReasonCode );

   /*--------------------------------------------------------+
   | If the disconnect failed, tell the user.                |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      /*--------------------------------------------------------+
      | Set the return and reason code in the API control area. |
      +--------------------------------------------------------*/

      printf( "MQDISC failed with Return Code = %d and Reason "\
               "Code = %d\n",
               MqmReturnCode,
               MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   return ( CW_RC );
}


#pragma subtitle ("CW_SendOneMqmMessage")
#pragma page ()

       /*----------------------------------------------------+
       | Routine: CW_SendOneMqmMessage                       |
       |                                                     |
       | Purpose: This routine will send a single MQM message|
       |          to the named queue. This call is different |
       |          from the normal MQM put in that the queue  |
       |          is not already open. The MQPUT1 call       |
       |          performs the open and close in the         |
       |          background.                                |
       |                                                     |
       | Input:                                              |
       |  QMgrName  - the name of the destination queue mgr. |
       |  QueueName - the name of the queue to be written to.|
       |  ReplyToQMgrName - the name of YOUR queue mgr.      |
       |  ReplyToQueueName- the name of YOUR reply queue.    |
       |  SendMsgPtr      - pointer to the send data block.  |
       |  SendMsgSize     - size of the send data blcok.     |
       |                                                     |
       | Output:                                             |
       |  None.                                              |
       |                                                     |
       | Return codes:                                       |
       |  CW_RC_OK     - The function completed              |
       |                  successfully.                      |
       |  CW_RC_FAILED - The function failed. The data was   |
       |                  not put onto a queue.              |
       |                                                     |
       | Abend codes:                                        |
       |  None.                                              |
       |                                                     |
       | Called Routines:                                    |
       |  MQPUT1                                             |
       |                                                     |
       +----------------------------------------------------*/

static CW_RC_T CW_SendOneMqmMessage( QUEUE_NAME     QMgrName,
                                     QUEUE_NAME     QueueName,
                                     QUEUE_NAME     ReplyToQMgrName,
                                     QUEUE_NAME     ReplyToQueueName,
                                     void          *SendMsgPtr,
                                     int            SendMsgSize )
{
   /*------------------------------------------------------+
   | Variable declarations.                                |
   +------------------------------------------------------*/

   auto   CW_RC_T CW_RC;        /* function return code        */

   auto   MQLONG    MqmReturnCode; /* return code from MQM        */

   auto   MQLONG    MqmReasonCode; /* reason code from MQM        */

   auto   MQOD      ObjDesc = { MQOD_DEFAULT };   /* MQOD block   */

   auto   MQPMO     PutMsgOpts = { MQPMO_DEFAULT };/* MQPMO block */

   auto   MQMD      MsgDesc = { MQMD_DEFAULT };   /* MQMD block   */

   /*------------------------------------------------------+
   | Constant definitions.                                 |
   +------------------------------------------------------*/

   /*------------------------------------------------------+
   | Code section                                          |
   +------------------------------------------------------*/

   printf("Now entering CW_SendOneMqmMessage\n");

   /*------------------------------------------------------+
   | Assume that the return code is OK.                    |
   +------------------------------------------------------*/

   CW_RC = CW_RC_OK;

   /*------------------------------------------------------+
   | Initialize the object name to the passed queue name.  |
   +------------------------------------------------------*/

   memcpy (ObjDesc.ObjectName,
           QueueName,
           MQ_Q_NAME_LENGTH);

   /*------------------------------------------------------+
   | Initialize the target queue manager to be the one     |
   | passed to us.                                         |
   +------------------------------------------------------*/

   memcpy (ObjDesc.ObjectQMgrName,
           QMgrName,
           MQ_Q_MGR_NAME_LENGTH);

   /*------------------------------------------------------+
   | Initialize the put message options to send the default|
   | context associated with the queue.                    |
   +------------------------------------------------------*/

   PutMsgOpts.Options = MQPMO_DEFAULT_CONTEXT;

   /*------------------------------------------------------+
   | Initialize the message descriptor. Set the reporting  |
   | to none.                                              |
   +------------------------------------------------------*/

   MsgDesc.Report = MQRO_NONE;

   /*------------------------------------------------------+
   | Set no expiration.                                    |
   +------------------------------------------------------*/

   MsgDesc.Expiry = MQEI_UNLIMITED;

   /*------------------------------------------------------+
   | Set the message type to be a request. This will allow |
   | the sender to determine who sent the request.         |
   +------------------------------------------------------*/

   MsgDesc.MsgType = MQMT_REQUEST;

   /*------------------------------------------------------+
   | default to the persistence and the priority of the    |
   | queue.                                                |
   +------------------------------------------------------*/

   MsgDesc.Persistence = MQPER_PERSISTENCE_AS_Q_DEF;
   MsgDesc.Priority    = MQPRI_PRIORITY_AS_Q_DEF;

   /*------------------------------------------------------+
   | Set the message id and correlation id to none. We just|
   | want the first one off the queue.                     |
   +------------------------------------------------------*/

   memcpy( MsgDesc.MsgId,
           MQMI_NONE,
           sizeof(MsgDesc.MsgId) );

   memcpy( MsgDesc.CorrelId,
           MQCI_NONE,
           sizeof(MsgDesc.CorrelId) );

   /*------------------------------------------------------+
   | Set the reply-to-queue name to the name passed by     |
   | the caller.                                           |
   +------------------------------------------------------*/

   memcpy ( MsgDesc.ReplyToQ,
            ReplyToQueueName,
            sizeof(MsgDesc.ReplyToQ) );

   /*------------------------------------------------------+
   | Set the reply-to-queue manager name to the local MQM  |
   | queue manager.                                        |
   +------------------------------------------------------*/

   memcpy ( MsgDesc.ReplyToQMgr,
            ReplyToQMgrName,
            sizeof(MsgDesc.ReplyToQMgr) );

   /*--------------------------------------------------------+
   | Call MQPUT1 to put the message onto the queue.          |
   +--------------------------------------------------------*/

   MQPUT1( LocalMqmHandle,
           &ObjDesc,
           &MsgDesc,
           &PutMsgOpts,
           SendMsgSize,
           SendMsgPtr,
           &MqmReturnCode,
           &MqmReasonCode );

   printf("After MQPUT1 - Mqm Return code =%d - Reason code %d\n",
              MqmReturnCode,
              MqmReasonCode );

   /*--------------------------------------------------------+
   | Examine the return code.                                |
   +--------------------------------------------------------*/

   if ( MqmReturnCode != MQCC_OK )
   {
      /*--------------------------------------------------------+
      | Set the return and reason code in the API control area. |
      +--------------------------------------------------------*/

      printf( "MQPUT1 failed with Return Code = %d and Reason Code "\
               "= %d\n",
               MqmReturnCode,
               MqmReasonCode );

      CW_RC = CW_RC_FAILED;
   }

   /*--------------------------------------------------------+
   | If the MQM put worked, issue a commit call to forward   |
   | the message to the target application.                  |
   +--------------------------------------------------------*/

   if ( CW_RC == CW_RC_OK )
   {
      printf("Now doing the commit.\n");

      MQCMIT( LocalMqmHandle,
              &MqmReturnCode,
              &MqmReasonCode );

      /*-----------------------------------------------------+
      | Examine the return code.                             |
      +-----------------------------------------------------*/

      if ( MqmReturnCode != MQCC_OK )
      {
         /*--------------------------------------------------------+
         | Set the return and reason code in the API control area. |
         +--------------------------------------------------------*/

         printf( "MQCMIT failed with Return Code = %d and Reason "\
                  "Code = %d\n",
                  MqmReturnCode,
                  MqmReasonCode );

         CW_RC = CW_RC_FAILED;
      }
   }

   return ( CW_RC );
}

#endif
